/***************************************************************************
                          callbacks.c  -  description
                             -------------------
    begin                : Tue Jul 4 2000
    copyright            : (C) 2000 by Simon E. Reichelt
    email                : s.reichelt@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdkprivate.h>
#include <malloc.h>
#include <stdio.h>
#include <string.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "texdat.h"
#include "wadview.h"

#include "../pixmaps/wv_logo.xpm"


short open_cmap = TRUE;

GtkWidget *cmap_win = NULL;

GtkWidget *cmap_canvas;


gboolean
ExitApp                                (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	gtk_main_quit();
  	
  	return FALSE;
}


void
InitWadLoading                         (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *fopen_d;
	
	//This callback function handles the
	//file selection dialog and proceeds to
	//the InitWadLoad() function
	
	fopen_d = gtk_file_selection_new("Select a WAD File");
	gtk_file_selection_set_filename(GTK_FILE_SELECTION(fopen_d),
	                                "~/");
	gtk_window_set_modal(GTK_WINDOW(fopen_d), TRUE);
	
	gtk_signal_connect_object(GTK_OBJECT(
	                          GTK_FILE_SELECTION(fopen_d)->cancel_button),
	                   "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
	                   (void*) fopen_d);
	
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fopen_d)->ok_button),
	                   "clicked", GTK_SIGNAL_FUNC(InitWadLoad), button);
	
	gtk_widget_show(fopen_d);
}


void
FlushTextures                          (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *scr_win;
	GtkWidget *m_win;
	GtkWidget *label1;
	GtkAdjustment *v_adj;
	
	if (!pck_table1)
		return;
		
	if (!open_cmap)  //check for open colormap window
	{
		gtk_widget_destroy(cmap_win);
		open_cmap = TRUE;
	}
	
	scr_win = lookup_widget(GTK_WIDGET(button), "scrolledwindow1");
	v_adj = gtk_scrolled_window_get_vadjustment(
	        GTK_SCROLLED_WINDOW(scr_win));
	
	gtk_adjustment_set_value(v_adj, 0);	
	v_adj->upper = 0;
	
	m_win = lookup_widget(GTK_WIDGET(button), "window1");
	
	label1 = lookup_widget(GTK_WIDGET(button),"label2");
	gtk_label_set_text(GTK_LABEL(label1), "0 x 0");
	
	gtk_widget_destroy(pck_table1);  //destroy browser table
	
	gtk_widget_unmap(scr_win);
	gtk_widget_map(scr_win);

	FreeTextures(tex_dat);  //free texture cache data
	
	tex_dat = NULL;	
	pck_table1 = NULL;
	
	sel_index = -1;  //set selection index to "unselected" value
	
	gtk_window_set_title(GTK_WINDOW(m_win), "WadVision v2.0");
}


/*void
WriteTexList                           (GtkButton       *button,
                                        gpointer         user_data)
{
	FILE *lst_file;
	GtkWidget *m_win;
	char f_name[1024];
	char *w_file;
	int i;
	int x_real, y_real;
	
	if (!tex_dat)
		return;
	
	m_win = gtk_widget_get_toplevel(GTK_WIDGET(button));
	
	strcpy(f_name, GTK_WINDOW(m_win)->title);
	
	w_file = strdup(f_name);
	
	f_name[strlen(f_name) - 4] = '\0';
	
	strcat(f_name, ".wdc");
	
	lst_file = OpenFile(f_name, "w");
	
	fprintf(lst_file, "TextureFile: %s\n\n", w_file);

	for (i = 0; i < tex_cnt - 1; i++)
	{
		GetRealSize(&tex_dat[i], &x_real, &y_real);
		
		fprintf(lst_file, "%s :   %d x %d\n",
		        tex_dat[i].name, x_real, y_real);
	}
	
	fclose(lst_file);
	
	ConPrint("WriteTexList(): texture list written.\n");
}*/


void JumpToTexture(GtkWidget *scr_win, int tex_i)
{
	GtkAdjustment *v_adj;
	int max_size, row_cnt;
	int tex_row, percent;
	int adj_pos;
	
	row_cnt = tex_cnt / 4;
	
	if ((tex_cnt % 4) > 0)
		row_cnt++;  //calculate table row count
		
	tex_row = (tex_i + 1) / 4;
	
	if (((tex_i + 1) % 4) > 0)
		tex_row++;  //calculate tex position row
	
	v_adj = gtk_scrolled_window_get_vadjustment(
	        GTK_SCROLLED_WINDOW(scr_win));
	
	max_size = v_adj->upper;  //get table height
	
	percent = tex_row * 100 / row_cnt;
	
	adj_pos = (max_size * percent / 100) - 270;
	
	gtk_adjustment_set_value(v_adj, adj_pos);
}


void
SearchTexture                          (GtkEditable     *editable,
                                        gpointer         user_data)
{
	int i, pat_len;
	int tex_index = -1;
	char s_pattern[16];
	GdkEvent b_event;
	gboolean bool_var;
	GtkWidget *scr_win;
	
	b_event.button.type = GDK_BUTTON_PRESS;
	b_event.button.button = 1;
	
	strcpy(s_pattern, gtk_entry_get_text(GTK_ENTRY(editable)));
	
	if ((pat_len = strlen(s_pattern)) == 0)
		return;
	
	for (i = 0; i < tex_cnt - 1; i++)
	{
		if (!strncasecmp(s_pattern, tex_dat[i].name, pat_len))
		{			
			tex_index = i;
			
			break;
		}
	}
	
	if (tex_index == -1)  //check for unchanged tex index
	{
		ConPrint("Search pattern not found.\n");
		
		return;
	}
	else if (tex_index == sel_index)
		return;
	
	MouseEvent((GtkWidget*) editable, (GdkEvent*) &b_event,
	          &tex_dat[tex_index]);  //simulate button click
	
	gtk_signal_emit_by_name(GTK_OBJECT(tex_dat[tex_index].d_area),
	                        "expose_event", (GtkWidget*) editable,
	                        &bool_var, NULL, &tex_dat[tex_index]);
	
	scr_win = lookup_widget(GTK_WIDGET(editable), "scrolledwindow1");
	
	JumpToTexture(scr_win, tex_index);
	
	gtk_widget_draw_focus(GTK_WIDGET(editable));
}


int RedrawCmap(GtkWidget *emitter, GdkEvent *event, gpointer data)
{
	GdkGC *gc;
	byte_t *pxm_dat;
	GdkRgbCmap *clr_map;
	int tex_i;
	
	gc = gdk_gc_new(emitter->window);
	
	//gdk_draw_pixmap(emitter->window, gc, tex_dat[tex_cnt - 1].pxm_data,
	//                0, 0, 0, 0, 256, 192);
	
	pxm_dat = tex_dat[tex_cnt - 1].pxm_data;
	clr_map = tex_dat[tex_cnt - 1].clr_map;
	
	if (wad_type == WAD3)
	{
		if (sel_index == -1)
			tex_i = 0;
		else
			tex_i = sel_index;
		
		pxm_dat = DrawCmapPixmap();
		
		clr_map = tex_dat[tex_i].clr_map;
	}
	
	gdk_draw_indexed_image(emitter->window, gc, 0, 0, 256, 192,
	                       GDK_RGB_DITHER_NORMAL,
	                       pxm_dat, 256, clr_map);
		
	return TRUE;
}


int CloseCmap(GtkWidget *emitter, GdkEvent *event, gpointer data)
{	
	gtk_widget_destroy(cmap_win);
	
	open_cmap = TRUE;
	cmap_win = NULL;
	
	return FALSE;
}


void ShowColormap(GtkButton *emitter, gpointer data)
{
	GtkWidget *win;
	GtkWidget *d_area;
	
	if (!tex_dat || !open_cmap)
		return;
	
	win = cmap_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(win),
	                     "Color Palette");
	gtk_window_set_policy(GTK_WINDOW(win), FALSE, FALSE,
	                      FALSE);
	gtk_signal_connect(GTK_OBJECT(win), "delete_event",
	                   GTK_SIGNAL_FUNC(CloseCmap), &open_cmap);
	
	cmap_canvas = d_area = gtk_drawing_area_new();
	gtk_drawing_area_size(GTK_DRAWING_AREA(d_area),
	                      256, 192);
	gtk_widget_set_events(d_area, GDK_EXPOSURE_MASK);
	gtk_signal_connect(GTK_OBJECT(d_area), "expose_event",
	                   GTK_SIGNAL_FUNC(RedrawCmap), NULL);
	
	gtk_container_set_border_width(GTK_CONTAINER(win),
	                               14);
	gtk_container_add(GTK_CONTAINER(win), d_area);
	
	gtk_widget_show(d_area);
	gtk_widget_show(win);
	
	open_cmap = FALSE;
}


void
CloseAboutDlg                          (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *dlg_win;
	
	dlg_win = gtk_widget_get_toplevel(GTK_WIDGET(button));
	
	gtk_widget_destroy(dlg_win);	
}


GtkWidget *CreateCachedPixmap(GtkWidget *widget, char *xpm_dat[])
{
	GdkPixmap *gdk_pxm;
	GtkWidget *gtk_pxm;
	GdkColormap *cmap;
	
	//Create a pixmap from virtual memory data.
	//Glade SHOULD do this for us :-)
	
	cmap = gtk_widget_get_colormap(widget);
	
	gdk_pxm = gdk_pixmap_colormap_create_from_xpm_d(NULL, cmap,
	                                       NULL, NULL, xpm_dat);
	
	gtk_pxm = gtk_pixmap_new(gdk_pxm, NULL);
	
	return GTK_WIDGET(gtk_pxm);
}


GtkWidget*
CreateAboutDlg (void)
{
  GtkWidget *window2;
  GtkWidget *frame3;
  GtkWidget *hbox3;
  GtkWidget *frame4;
  GtkWidget *pixmap1;
  GtkWidget *vbox2;
  GtkWidget *frame5;
  GtkWidget *label5;
  GtkWidget *hbox4;
  GtkWidget *button9;

  window2 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window2), "About WadVision");
  gtk_window_set_policy (GTK_WINDOW (window2), FALSE, FALSE, FALSE);

  frame3 = gtk_frame_new ("Info Panel");
  gtk_widget_show (frame3);
  gtk_container_add (GTK_CONTAINER (window2), frame3);
  gtk_container_set_border_width (GTK_CONTAINER (frame3), 10);
  gtk_frame_set_label_align (GTK_FRAME (frame3), 0.03, 0.5);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox3);
  gtk_container_add (GTK_CONTAINER (frame3), hbox3);

  frame4 = gtk_frame_new (NULL);
  gtk_widget_show (frame4);
  gtk_box_pack_start (GTK_BOX (hbox3), frame4, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame4), 8);
  gtk_frame_set_shadow_type (GTK_FRAME (frame4), GTK_SHADOW_IN);

  pixmap1 = CreateCachedPixmap(window2, wv_logo_xpm);
  gtk_widget_show (pixmap1);
  gtk_container_add (GTK_CONTAINER (frame4), pixmap1);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (hbox3), vbox2, FALSE, TRUE, 0);

  frame5 = gtk_frame_new ("WadVision v2.0");
  gtk_widget_show (frame5);
  gtk_box_pack_start (GTK_BOX (vbox2), frame5, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame5), 8);
  gtk_frame_set_label_align (GTK_FRAME (frame5), 0.15, 0.5);

  label5 = gtk_label_new ("Copyright (C) 2000\nby\nSimon E. Reichelt");
  gtk_widget_show (label5);
  gtk_container_add (GTK_CONTAINER (frame5), label5);
  gtk_widget_set_usize (label5, 130, -2);
  gtk_misc_set_alignment (GTK_MISC (label5), 0.5, 0.45);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox4, FALSE, FALSE, 42);

  button9 = gtk_button_new_with_label ("OK");
  gtk_widget_show (button9);
  gtk_box_pack_start (GTK_BOX (hbox4), button9, TRUE, FALSE, 0);
  gtk_widget_set_usize (button9, 70, -2);
  GTK_WIDGET_UNSET_FLAGS (button9, GTK_CAN_FOCUS);
  GTK_WIDGET_SET_FLAGS (button9, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (button9), "clicked",
                      GTK_SIGNAL_FUNC (CloseAboutDlg),
                      NULL);

  gtk_widget_grab_default (button9);
  return window2;
}


void
CreateAboutDialog                      (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *about;
		
	about = CreateAboutDlg();
	
	gtk_window_set_modal(GTK_WINDOW(about), TRUE);
	
	gtk_widget_show(about);
}